#!/usr/bin/perl -w
#
# $Date: 2007-12-27 12:42:22 $ , $Revision: 1.5 $
# (c) Ruedi Hofer

#use strict;
#use warnings;
use Getopt::Long;
use Image::ExifTool 'ImageInfo';

use vars qw/ %optctl $verbose $test $getOptRes/;

$title="";
@globalArr=();

$verbose=1;
$test=1;
$increment=1;
$reset=0;

$getOptRes=GetOptions("verbose=i" => \$verbose,
                      "increment=i" => \$increment,
                      "reset=i" => \$reset,
                      "test=i"    => \$test);

if ($getOptRes == 0 || $#ARGV != 0) {
   print "Usage: renameImage [-v|--verbose] [-t|--test] [-i|--increment] [-r|--reset] <dir>\n\n";
   print "Rename image filenames according to exif date/time in the form 0000_YYMMDD_HHMM_title.jpg, where 0000 is a serial number.\n";
   print "The serial number was introduced to change the file order for improved slideshow viewing.\n\n";
   print "Special Features:\n";
   print "- Images from different cameras get sorted by date and numbered according to the scheme.\n";
   print "- If non-exif-files (wav, avi..) are detected, they are inserted after the last known image.\n";
   print "- The filenames of renamed images don't get altered unless -r is used.\n";
   print "- New and not yet renamed images get inserted after the first appropriate position by date.\n\n";
   print "Options:\n";
   print "-v|--verbose\n";
   print "-t|--test\n";
   print "-i|--increment    Increment the counter.\n";
   print "-r|--reset        Reset the order, start numbering according to the date.\n";
   exit 0;
}

print "Modus   test=$test, verbose=$verbose, increment=$increment, reset=$reset\n" if $verbose;

opendir my $dir, "$ARGV[0]" or die "Cannot open directory: $!";
@files = sort grep { -f "$ARGV[0]/$_" } readdir $dir;
closedir $dir or die "Cannot close .: $!";
#print @files; exit;

$lastEntry=0;

to_globalArr($_) for (@files);

if ($test) {
  print_globalArr();
} else {
  rename_globalArr();
}

exit 1;

sub to_globalArr {
   ($img) = @_;
   
   $prefix = $img;
   $prefix =~ s/.*\.//g;
   $imgArr[0] = $img;

   print "Image $ARGV[0]/$img\n" if $verbose && $test;

   # Erst kommen immer die files mit einer Nummer vorne, dh die welche schon sortiert sind. 
   # Diese wollen wir so behalten!!
   $img =~ /^(\d\d\d\d)_?(.*)\.(.*)$/;
   if ($1 && !$reset) {
   	 $title = $2;
   	 $prefix = $3;

	 if ($3 && ($3 eq 'jpeg' || $3 eq 'jpg')) {
	   $ts = ImageInfo("$ARGV[0]/$img", 'DateTimeOriginal')->{DateTimeOriginal};
	   if (defined $ts) {
	     $ts =~ /^(\d\d)(\d\d):(\d\d):(\d\d)\s(\d\d):(\d\d):(\d\d)$/ ;
	     $imgArr[1] = sprintf ("%s%s%s%#02s%s",$2,$3,$4,$5,$6);
	   }
	 } else {
	   $imgArr[1] = "0000000000";   	
     }

     $imgArr[2] = sprintf ("%s.%s", $title, $prefix);
   	 push (@globalArr, [ @imgArr ]);
     print "  > imgArr added at end $#globalArr. Renamed or manually moved img found." if $verbose && $test;

     $lastEntry=$#globalArr+1;
     print " lastEntry=$lastEntry.\n" if $verbose && $test;

   	 return;
   }


   # Wenn das format stimmt dann wird $3 gesetzt
   $img =~ /^(.*)(\d\d\d\d)_?(.*)\.(.*)$/;
   if ($3) {
     $title = $3;
   } else {
     $title = "title";
   }

   $inserted=0;
   if ($4 && ($4 eq 'jpeg' || $4 eq 'jpg')) {
      $ts = ImageInfo("$ARGV[0]/$img", 'DateTimeOriginal')->{DateTimeOriginal};

      if (defined $ts) {
         $ts =~ /^(\d\d)(\d\d):(\d\d):(\d\d)\s(\d\d):(\d\d):(\d\d)$/ ;
         $imgArr[1] = sprintf ("%s%s%s%#02s%s",$2,$3,$4,$5,$6);
         $imgArr[2] = sprintf ("%s%s%s_%#02s%s_%s.%s",$2,$3,$4,$5,$6,$title,$prefix);
      }
      else {
         print "  !! ERROR on $img - no exif data !!\n";
      }
   }
   else {
   	  # addiere File ohne Exif Kommentar hinter das zuletzt eingefügte Element
      $imgArr[1] = "0000000000";
      $imgArr[2] = sprintf ("%s.%s", $title, $prefix);
      splice @globalArr,$lastEntry,$#globalArr-$lastEntry,([@imgArr],@globalArr[$lastEntry..($#globalArr+-1)]);
      print "  > imgArray added at lastEntry $lastEntry (of $#globalArr)." if $verbose && $test;
      $lastEntry++;
      $inserted=1;
   }


   
   # in globalArr einfügen
   for ($i=0; $i<= $#globalArr && $inserted==0; $i++) {
     print "    Compare with entry $i $globalArr[$i][1] > $imgArr[1]\n" if $verbose && $test;
     if ($globalArr[$i][1]>$imgArr[1]) {
       splice @globalArr,$i,$#globalArr-$i,([@imgArr],@globalArr[$i..($#globalArr+-1)]);
       print "  > imgArray added at position $i (of $#globalArr)." if $verbose && $test;
       $inserted=1;
     }
     $lastEntry=$i+1;
   }
   
   # wenn array leer oder wenn zuhinterst eingefügt werden muss
   if($#globalArr==-1) {
   	 push (@globalArr, [ @imgArr ]);
     $lastEntry=1;
     print "  > imgArray added at position $#globalArr (first)." if $verbose && $test;
   } elsif($inserted==0) {
   	 push (@globalArr, [ @imgArr ]);
     $lastEntry=$#globalArr+1;
     print "  > imgArray added at position $#globalArr (end)." if $verbose && $test;
   }
   print " lastEntry=$lastEntry.\n" if $verbose && $test;
}

sub print_globalArr {
   print"\nList\n";
   for ($i=0; $i<= $#globalArr; $i++) {
     $out=sprintf ("%#04s_%s",$i*$increment,$globalArr[$i][2]);
     printf ("  %03i %#-30s > %#-30s (%s)\n", $i, $globalArr[$i][0],$out,$globalArr[$i][1]);
   }
}

sub rename_globalArr {
   print"\nRename\n" if $verbose;
   for ($i=0; $i<= $#globalArr; $i++) {
     $out=sprintf ("%#04s_%s",$i*$increment,$globalArr[$i][2]);
     printf ("  %03i %#-30s > %#-30s (%s)\n", $i, $globalArr[$i][0],$out,$globalArr[$i][1]);
     rename "$ARGV[0]/$globalArr[$i][0]", "$ARGV[0]/$out";
   }
}
